----------------------------------------------------------------------------------
-- Company: Digilent RO
-- Engineer: Ioana Dabacan
-- 
-- Create Date:    17:01:53 03/03/2008 
-- Design Name: 
-- Module Name:    CON3RefProj - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: This document describes the VHDL implementation of the Digilent 
--             CON3 Reference Project. The project instantiates a Digilent CON3
--             Reference Component which controls the pins to drive four Servo
--             motors. The orientation of each Servo motor can be set from a
--             couple of buttons, one for moving it to the left and the other
--             one for moving it to the right. Only one Servo can be positioned
--             at one time. The channel for the current Servo is selected from
--             another button. The seven segment display shows the channel
--             number and the Pulse Width value of the signal used to drive the
--             corresponding Servo. 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity CON3RefProj is
 Port ( 
           ck : in  STD_LOGIC;            -- system clock (50MHz)
           btnCntUp : in  STD_LOGIC;      -- rise Pulse Width command
           btnCntDown : in  STD_LOGIC;    -- lower Pulse Width command
           btnChannel : in  STD_LOGIC;    -- channel selector
           pinCtrlA : out  STD_LOGIC;     -- Servo A Control pin 
           pinCtrlB : out  STD_LOGIC;     -- Servo B Control pin
           pinCtrlC : out  STD_LOGIC;     -- Servo C Control pin 
           pinCtrlD : out  STD_LOGIC;     -- Servo D Control pin 
           seg: out std_logic_vector(6 downto 0); -- segment outputs
           an : out std_logic_vector(3 downto 0));-- anode select signals
			  
end CON3RefProj;

architecture Behavioral of CON3RefProj is

COMPONENT CON3RefComp
	PORT(
           ck : in  STD_LOGIC;  
           vecPwInA : in  STD_LOGIC_VECTOR(7 downto 0); 
           vecPwInB : in  STD_LOGIC_VECTOR(7 downto 0); 
           vecPwInC : in  STD_LOGIC_VECTOR(7 downto 0); 
	     vecPwInD : in  STD_LOGIC_VECTOR(7 downto 0);
           bitCtrlOutA : out  STD_LOGIC;  
           bitCtrlOutB : out  STD_LOGIC;   
           bitCtrlOutC : out  STD_LOGIC;   
           bitCtrlOutD : out  STD_LOGIC
		);
	END COMPONENT;
	
  signal vecPwA: STD_LOGIC_VECTOR (7 downto 0) := x"00"; -- Pulse Width for PWM (ch A)
  signal vecPwB: STD_LOGIC_VECTOR (7 downto 0) := x"00"; -- Pulse Width for PWM (ch B)
  signal vecPwC: STD_LOGIC_VECTOR (7 downto 0) := x"00"; -- Pulse Width for PWM (ch C)
  signal vecPwD: STD_LOGIC_VECTOR (7 downto 0) := x"00"; -- Pulse Width for PWM (ch D)
  signal vecPw: STD_LOGIC_VECTOR (7 downto 0) := x"00"; --  PW for selected channel
  signal vecGenPw: STD_LOGIC_VECTOR (7 downto 0) := x"00"; --  Generated PW
  signal vecCh : STD_LOGIC_VECTOR (3 downto 0) := x"1"; --  selected channel 
  signal hex : STD_LOGIC_VECTOR (4 downto 0); --  data for the SSD
  signal bitTmpPulse: std_logic := '0';--intermediary signal to generate 1 clock pulse
  signal bitPulse: std_logic := '0';--1 period clock period pulse for each button push
  signal cntSSD: std_logic_vector(20 downto 0):="000000000000000000000";-- counter for 7SD
  signal ckInt: std_logic := '0';--internal clock signal (23.84Hz)
  
begin

Inst_CON3RefComp: CON3RefComp PORT MAP(
		ck => ck,
		vecPwInA => vecPwA,
		vecPwInB => vecPwB,
		vecPwInC => vecPwC,
		vecPwInD => vecPwD,
		bitCtrlOutA => pinCtrlA,
		bitCtrlOutB => pinCtrlB,
		bitCtrlOutC => pinCtrlC,
		bitCtrlOutD => pinCtrlD
	);
--clock divider	
  div: process (ck)
    begin
     if rising_edge(ck) then
      cntSSD <= cntSSD + "0000000000000000001";
     end if;
  end process;

ckInt <=cntSSD(20);

with cntSSD(18 downto 17) select
 an <= "1110" when "00",
       "1101" when "01",
       "1011" when "10",
       "0111" when others; 
	 
with cntSSD(18 downto 17) select
 hex <= vecPw(3 downto 0)&'0' when "00",
        vecPw(7 downto 4)&'0' when "01",
		  "11111"  when "10",  -- nothing
        vecCh&'0' when others; -- Channel number

		  --HEX-to-seven-segment decoder
-- 
-- segment encoding
--      0
--     ---  
--  5 |   | 1
--     ---   <- 6
--  4 |   | 2
--     ---
--      3
   
    with HEX select
   seg<= "1000000" when "00000",   --0
         "1111001" when "00010",   --1
         "0100100" when "00100",   --2
         "0110000" when "00110",   --3
         "0011001" when "01000",   --4
         "0010010" when "01010",   --5
         "0000010" when "01100",   --6
         "1111000" when "01110",   --7
         "0000000" when "10000",   --8
         "0010000" when "10010",   --9
         "0001000" when "10100",   -- A
			"0000011" when "10110",   -- B
			"1000110" when "11000",   -- C
			"0100001" when "11010",   -- D
			"0000110" when "11100",   -- E
			"0001110" when "11110",   -- F
			"0111111" when others;   -- minus sign
         


   -- generate the pulse (active for one clock period on '1' logic)
   pulse: process(ckInt)
   begin
      if(rising_edge(ckInt)) then
         -- if input signal (btnChannel) goes low
         -- reset output and intermediary signal
         if(btnChannel = '0') then
            bitTmpPulse <= '0';
            bitPulse <= '0';
         -- if intermediary signal is low, this is the first
         -- clock period input goes high
         -- put output high
         -- else, this is not the first clock period input
         -- is high, so reset output.
         else
            if(bitTmpPulse = '0') then
               bitTmpPulse <= '1';
               bitPulse <= '1';
            else
               bitPulse <= '0';
            end if;
         end if;
      end if;
   end process pulse;

--generates the corresponding Pulse Width of the selected
--channel from the vecCh vector. When button  btnCntUp is '1'
--the Pulse Width increases and when the btnCntDown is '1' the 
--Pulse Width decreases for the slected channel. 
generatePulseWidth : process(ckInt)
  begin		  
    if(rising_edge(ckInt)) then
--increase or deacrease the Pulse Width value
      if (btnCntUp = '1'  and vecPw < x"96") then
						vecPw <= vecPw + '1';
		  elsif (btnCntDown = '1'  and vecPw > x"00") then
						vecPw <= vecPw - '1';	
	     elsif(bitPulse = '1')then
--load the value from the selected channel
		   if(vecCh = x"4")then
		        vecCh <= x"1";
				  vecPw <= vecPwA;
			    elsif vecCh = x"1" then 
				  vecCh <= vecCh + '1';
				  vecPw <= vecPwB;
				 elsif vecCh = x"2" then 
				   vecCh <= vecCh + '1';
               vecPw <= vecPwC;
				 else-- vecCh = x"3" then 
					vecCh <= vecCh + '1';
					vecPw <= vecPwD;
			  end if;
        end if;
--store back the selected value for each channel	  
     if(vecCh = x"1")then
       vecPwA <= vecPw;
	  elsif vecCh = x"2" then 
	    vecPwB <= vecPw;
	  elsif vecCh = x"3" then 
       vecPwC <= vecPw;
	  else-- vecCh = x"3" then 
			     vecPwD <= vecPw;
	  end if;
  end if;
end process;


end Behavioral;

